from .custom_type import PathType
from .array_profile import profile
from .array_threshold import compute_thresholds
from .helper_mask import (
    get_characteristics,
    significant_slice_idx,
    significant_slice_idx_data,
    mask_path_to_data_ax,
    mask_path_to_data_sag,
    image_path_to_data_ax,
    image_path_to_data_sag,
    mask_expand,
    convert_mask,
)


# from utility.log import logger_init
# from utility.reader import *
# from utility.utility import *



from pathlib import Path

import pandas as pd

import medviz as viz


def generate_mask_schema(
    dataset_path,
    id_func=lambda x: x,
    mask_format=".nii",
    output_path=r"./",
    output_name="mask_schema.csv",
):
    df_dict = {
        "id": [],
        "path": [],
        "shape": [],
        "data_type": [],
        "values": [],
        "mask_type": [],
        "most_value_nonzero_slices": [],
        "num_nonzero_slices": [],
    }

    mask_paths = dataset_path.glob(rf"**/*{mask_format}")

    for mask_path in mask_paths:
        # try:
        id = id_func(mask_path.stem)
        print(id)
        mask_characteristics = viz.get_characteristics(mask_path)
        most_value_nonzero_slices, num_nonzero_slices = viz.significant_slice_idx(
            mask_path
        )
        df_dict["id"].append(id)
        df_dict["path"].append(Path(*mask_path.parts[-3:]))
        df_dict["shape"].append(mask_characteristics["shape"])
        df_dict["data_type"].append(mask_characteristics["data_type"])
        df_dict["values"].append(mask_characteristics["values"])
        df_dict["mask_type"].append(mask_characteristics["mask_type"])
        df_dict["most_value_nonzero_slices"].append(most_value_nonzero_slices)
        df_dict["num_nonzero_slices"].append(num_nonzero_slices)

    # except Exception as e:
    #     print(f"Error in {mask_path}", e)

    df = pd.DataFrame(df_dict, columns=df_dict.keys())

    output_path = Path(output_path)
    if not output_path.exists():
        output_path.mkdir(parents=True)

    save_path = Path(output_path) / Path(output_name)
    df.to_csv(save_path, index=False)

    print("Done!")



from pathlib import Path

import matplotlib.pyplot as plt
from matplotlib.widgets import Slider

import medviz as viz

assert_shape = viz.assert_shape

image_path_to_data = viz.image_path_to_data_ax
mask_path_to_data = viz.mask_path_to_data_ax

generate_mask_colors = viz.generate_mask_colors
plot_image = viz.plot_image
plot_contour = viz.plot_contour


def layered_plot_path3D(
    image_path, mask_paths, mask_colors=None, title="Layered Plot", save_path=None
):
    image_data = image_path_to_data(image_path)

    masks_data = []
    for mask_path in mask_paths:
        mask_data = mask_path_to_data(mask_path)
        masks_data.append(mask_data)

    layered_plot_data3D(
        image_data,
        masks_data,
        mask_colors=mask_colors,
        title=title,
        save_path=save_path,
    )


def layered_plot_data3D(
    image_data, masks_data, mask_colors=None, title="Layered Plot", save_path=None
):
    print("Loading images...")

    num_masks = len(masks_data)

    init_slice, last_slice = assert_shape(image_data + masks_data)

    mask_colors = generate_mask_colors(num_masks, mask_colors)

    _, ax = plt.subplots()
    plt.subplots_adjust(bottom=0.25)

    plot_image(ax, image_data[:, :, init_slice], cmap="gray")

    for i in range(num_masks):
        plot_contour(
            ax, masks_data[i][:, :, init_slice], color=mask_colors[i], levels=[0.5]
        )

    ax.set_xlabel(f"Slice Number: {init_slice}")
    ax.set_title(title)

    slider_ax = plt.axes([0.2, 0.1, 0.6, 0.03])

    slider = Slider(
        slider_ax,
        "Slice",
        0,
        last_slice,
        valinit=init_slice,
        valstep=1,
    )

    def update(val):
        slice_num = int(slider.val)
        ax.clear()

        plot_image(ax, image_data[:, :, slice_num], cmap="gray")
        for i in range(num_masks):
            plot_contour(ax, masks_data[i][:, :, slice_num], color=mask_colors[i])

        ax.set_xlabel(f"Slice Number: {slice_num}")
        ax.set_title(title)

    slider.on_changed(update)

    plt.show()

    if save_path:
        if isinstance(save_path, str):
            save_path = Path(save_path)

        if save_path.suffix != ".png":
            save_path = save_path.with_suffix(".png")

        if not save_path.parent.exists():
            save_path.parent.mkdir(parents=True)

        plt.savefig(save_path)



from typing import List

import numpy as np
from tabulate import tabulate


def profile(arrays: List[np.ndarray], unique_values: bool = False) -> None:
    table = [
        [
            "ID",
            "Shape",
            "Dimensions",
            "Data Type",
            "Size (#Elements)",
            "Total Memory Size",
            "Max Value",
            "Min Value",
            "Sum of Values",
            "Mean Value",
            "Standard Deviation",
            "Median Value",
        ]
    ]

    if unique_values:
        table[0] += [
            "Unique Values",
            "Count of Each Unique Value",
            "Dictionary of Values and Counts",
        ]

    for i, arr in enumerate(arrays):
        row = [
            i,
            arr.shape,
            arr.ndim,
            arr.dtype,
            arr.size,
            arr.size * arr.itemsize,
            arr.max(),
            arr.min(),
            arr.sum(),
            arr.mean(),
            arr.std(),
            np.median(arr),
        ]

        if unique_values:
            unique, counts = np.unique(arr, return_counts=True)
            values = dict(zip(unique, counts))
            row += [unique, counts, values]

        table.append(row)

    print(tabulate(table, tablefmt="grid"))


def threshold(arrays: List[np.ndarray], threshold: int) -> None:
    table = [
        [
            "ID",
            "Shape",
            "Dimensions",
            "Data Type",
            "Size (#Elements)",
            "Total Memory Size",
            "Max Value",
            "Min Value",
            "Sum of Values",
            "Mean Value",
            "Standard Deviation",
            "Median Value",
        ]
    ]

    for i, arr in enumerate(arrays):
        arr[arr < threshold] = 0
        row = [
            i,
            arr.shape,
            arr.ndim,
            arr.dtype,
            arr.size,
            arr.size * arr.itemsize,
            arr.max(),
            arr.min(),
            arr.sum(),
            arr.mean(),
            arr.std(),
            np.median(arr),
        ]

        table.append(row)

    print(tabulate(table, tablefmt="grid"))



import logging
from os import getenv
from pathlib import Path

from .custom_type import PathType
from .utility import now


def logger_init(file_name: str):
    log_root = getenv("PATH_LOGS")

    path_logs: PathType = Path(log_root) if log_root != None else Path("./logs")

    if not path_logs.exists():
        path_logs.mkdir(parents=True)

    current_time = now()

    logging.basicConfig(
        filename=rf"{path_logs}/{file_name}_{current_time}.log",
        format="%(asctime)s %(message)s",
        encoding="utf-8",
        level=logging.DEBUG,
        datefmt="%Y-%m-%d %H:%M:%S",
    )

    logger = logging.getLogger(file_name)
    logger.info("Start.")

    return logger



import math

import matplotlib.pyplot as plt
import numpy as np
from matplotlib.widgets import Slider

from ...utils import image_path_to_data_ax
from .. import plot_image_imshow


def images_path(paths, rows=None, columns=None, titles=[], cmap="gray"):
    images_data = [image_path_to_data_ax(path) for path in paths]

    images_array(
        images_data=images_data,
        rows=rows,
        columns=columns,
        titles=titles,
        cmap=cmap,
    )


def images_array(images_data, rows=None, columns=None, titles=[], cmap="gray"):
    print("Loading images...")

    d = images_data[0].shape[2]
    init_slice, last_slice = d // 2, d - 1

    num_images = len(images_data)  # Number of images
    rows = math.ceil(math.sqrt(num_images))  # Number of rows in the grid
    columns = math.ceil(num_images / rows)  # Number of columns in the grid

    _, axs = plt.subplots(rows, columns)
    if num_images == 1:
        axs = np.array([axs])

    plt.subplots_adjust(bottom=0.25)

    for i, ax in enumerate(axs.flat):
        if i < num_images:
            title = titles[i] if titles else f"Image {i}"
            plot_image_imshow(
                ax, images_data[i][:, :, init_slice], cmap=cmap, title=title
            )
            ax.axis("off")
            ax.set_xlabel(f"Slice Number: {init_slice}")

        else:
            ax.axis("off")

    slider_ax = plt.axes([0.2, 0.1, 0.6, 0.03])

    slider = Slider(
        slider_ax,
        "Slice",
        0,
        last_slice,
        valinit=init_slice,
        valstep=1,
    )

    def update(val):
        slice_num = int(slider.val)

        for i, ax in enumerate(axs.flat):
            ax.clear()

            if i < num_images:
                title = titles[i] if titles else f"Image {i}"

                plot_image_imshow(
                    ax, images_data[i][:, :, slice_num], cmap=cmap, title=title
                )
                ax.set_xlabel(f"Slice Number: {slice_num}")
                ax.axis("off")
            else:
                ax.axis("off")

        # ax.set_title(title)

    slider.on_changed(update)

    plt.tight_layout()
    plt.show()


# import nibabel as nib

# if __name__ == "__main__":
#     ct_image = nib.load("dataset/1-1.nii")
#     ct_data = ct_image.get_fdata()
#     print(ct_data.shape)
#     ct_data = image_path_to_data("dataset/1-1.nii")
#     # ct_data = ct_data[:, :, 100]
#     plot_multi_data([ct_data, ct_data], title="Image")



from os import getenv
from pathlib import Path

from .custom_type import PathType


def path_in(p: str, global_path=True) -> PathType:
    # if p[0] == "/" or not global_path:  # Check whether p is absolute path.
    #     path_input = p
    # else:
    #     root_in = getenv("ROOT_IN") or "."
    #     path_input = rf"{root_in}/{p}"

    # path: PathType = Path(path_input)
    # if not path.exists():
    #     raise Exception(f"{p} doesn't exist on the earth.")

    path = p

    return path


def path_out(p: str, global_path=True) -> PathType:
    if p[0] == "/" or not global_path:  # Check whether p is absolute path
        path_output = p
    else:
        root_out = getenv("ROOT_OUT") or "."
        path_output = rf"{root_out}/{p}"

    path: PathType = Path(path_output)
    if not path.exists():
        print("The output path doesn't exist but no worries I'm making it.")
        path.mkdir(parents=True)

    return path



import math

import matplotlib.pyplot as plt
import numpy as np

from ...plots import plot_image_imshow, save_image
from ...utils import image_path_to_data_ax


def images_path(paths, rows=None, columns=None, titles=[], cmap="gray", save_path=None):
    images_data = [
        image_path_to_data_ax(path) for path in paths
    ]  # just for nifti files

    images_array(
        images_data=images_data,
        rows=rows,
        columns=columns,
        titles=titles,
        cmap=cmap,
        save_path=save_path,
    )


def images_array(
    images_data, rows=None, columns=None, titles=[], cmap="gray", save_path=None
):
    print("Loading images...")

    num_images = len(images_data)  # Number of images
    rows = math.ceil(math.sqrt(num_images))  # Number of rows in the grid
    columns = math.ceil(num_images / rows)  # Number of columns in the grid

    _, axs = plt.subplots(rows, columns)
    if num_images == 1:
        axs = np.array([axs])

    for i, ax in enumerate(axs.flat):
        if i < num_images:
            title = titles[i] if titles else f"Image {i}"
            plot_image_imshow(ax, images_data[i], cmap=cmap, title=title)
            ax.axis("off")
        else:
            ax.axis("off")

    plt.tight_layout()

    if save_path:
        save_image(plt, save_path)
    else:
        plt.show()



# DocString: Create a gif from a 3D image and multiple 3D masks
"""
Create a gif from a 3D image and multiple 3D masks

Parameters
----------
image_path : str
    Path to the 3D image
mask_paths : list
    List of paths to the 3D masks
mask_colors : list  
    List of colors for each mask
interval : int, optional    
    Interval between frames in milliseconds (default is 100)
start_slice : int, optional
    Starting slice (default is 0)
end_slice : int, optional
    Ending slice (default is None)
title : str, optional
    Title of the gif (default is "Layered Plot")
save_path : str, optional
    Path to save the gif (default is "animation.gif")

Returns
-------
None

"""

from pathlib import Path

import matplotlib.animation as animation
import matplotlib.pyplot as plt
import numpy as np

import medviz as viz

assert_shape = viz.assert_shape

image_path_to_data = viz.image_path_to_data_ax
mask_path_to_data = viz.mask_path_to_data_ax

generate_mask_colors = viz.generate_mask_colors

mask_expand = viz.mask_expand

# def assert_shape(arr):
#     # image shape
#     n, m, d = arr[0].shape
#     try:
#         for np_array in arr:
#             assert np_array.shape == (m, n, d)
#     except AssertionError:
#         raise ValueError("All segments must have the same shape as the image")

#     return d // 2, d - 1


def gif(
    image_path,
    mask_paths,
    mask_colors=None,
    interval=100,
    start_slice=0,
    end_slice=None,
    title="",
    segments=None,
    save_path=None,
):
    ct_data = image_path_to_data(image_path)

    masks_data = []
    for mask_path in mask_paths:
        mask_data = mask_path_to_data(mask_path)
        masks_data.append(mask_data)

    num_masks = len(masks_data)

    if segments:
        try:
            assert len(segments) == num_masks
        except AssertionError:
            # raise ValueError("Number of segments must be equal to the number of masks")
            print(
                "Number of segments must be equal to the number of masks, segments will be ignored"
            )
            segments = None

    # _, last_slice = assert_shape(ct_data + masks_data)
    last_slice = ct_data.shape[-1] - 1

    mask_colors = generate_mask_colors(num_masks, mask_colors)

    fig = plt.figure()

    end_slice = last_slice if end_slice is None else end_slice

    slice_range = range(start_slice, end_slice)

    def update_slice(frame):
        global current_slice

        current_slice = frame

        plt.clf()

        plt.imshow(ct_data[:, :, current_slice], cmap="gray")

        xbar_title = ""

        for i in range(len(masks_data)):
            if not np.all(masks_data[i][:, :, current_slice] == False):
                xbar_title_update = (
                    f"{segments[i]}: {mask_colors[i]}"
                    if segments
                    else f"Mask {i}: {mask_colors[i]}"
                )
                xbar_title = xbar_title + xbar_title_update

            plt.contour(
                masks_data[i][:, :, current_slice],
                colors=mask_colors[i],
                levels=[0.5],
            )

        plt.title(f"{title} Slice {current_slice} {xbar_title}")
        plt.axis("off")

    ani = animation.FuncAnimation(
        fig, update_slice, frames=slice_range, interval=interval
    )

    if save_path:
        if isinstance(save_path, str):
            save_path = Path(save_path)

        if save_path.suffix != ".png":
            save_path = save_path.with_suffix(".gif")

        if not save_path.parent.exists():
            save_path.parent.mkdir(parents=True)

        ani.save(save_path, writer="pillow")

    plt.show()


def gif_slices(
    image_path,
    mask_paths,
    slices,
    mask_colors=None,
    interval=100,
    title="",
    segments=None,
    save_path=None,
):
    ct_data = image_path_to_data(image_path)

    masks_data = []
    for mask_path in mask_paths:
        mask_data = mask_path_to_data(mask_path)
        masks_data.append(mask_data)

    num_masks = len(masks_data)

    if segments:
        try:
            assert len(segments) == num_masks
        except AssertionError:
            # raise ValueError("Number of segments must be equal to the number of masks")
            print(
                "Number of segments must be equal to the number of masks, segments will be ignored"
            )
            segments = None

    mask_colors = generate_mask_colors(num_masks, mask_colors)

    fig = plt.figure()

    def update_slice(frame):
        global current_slice

        current_slice = frame

        plt.clf()

        plt.imshow(ct_data[:, :, current_slice], cmap="gray")

        xbar_title = ""

        for i in range(len(masks_data)):
            if not np.all(masks_data[i][:, :, current_slice] == False):
                xbar_title_update = (
                    f"{segments[i]}: {mask_colors[i]}"
                    if segments
                    else f"Mask {i}: {mask_colors[i]}"
                )
                xbar_title = xbar_title + xbar_title_update

            plt.contour(
                masks_data[i][:, :, current_slice],
                colors=mask_colors[i],
                levels=[0.5],
            )

        plt.title(f"{title} Slice {current_slice} {xbar_title}")
        plt.axis("off")

    ani = animation.FuncAnimation(fig, update_slice, frames=slices, interval=interval)

    if save_path:
        if isinstance(save_path, str):
            save_path = Path(save_path)

        if save_path.suffix != ".png":
            save_path = save_path.with_suffix(".gif")

        if not save_path.parent.exists():
            save_path.parent.mkdir(parents=True)

        ani.save(save_path, writer="pillow")

    plt.show()


def gif_non(
    image_path,
    mask_paths,
    mask_colors=None,
    interval=100,
    title="",
    segments=None,
    save_path=None,
):
    ct_data = image_path_to_data(image_path)

    masks_data = []
    slices = set()
    for mask_path in mask_paths:
        mask_data = mask_path_to_data(mask_path)
        most_value_nonzero_slices, _ = viz.significant_slice_idx_data(mask_data)
        slices.update(most_value_nonzero_slices)
        masks_data.append(mask_data)

    num_masks = len(masks_data)

    if segments:
        try:
            assert len(segments) == num_masks
        except AssertionError:
            # raise ValueError("Number of segments must be equal to the number of masks")
            print(
                "Number of segments must be equal to the number of masks, segments will be ignored"
            )
            segments = None

    # assert_shape(ct_data + masks_data)

    mask_colors = generate_mask_colors(num_masks, mask_colors)

    fig = plt.figure()

    slice_range = slices

    def update_slice(frame):
        global current_slice

        current_slice = frame

        plt.clf()

        plt.imshow(ct_data[:, :, current_slice], cmap="gray")

        xbar_title = ""

        for i in range(len(masks_data)):
            if not np.all(masks_data[i][:, :, current_slice] == False):
                xbar_title_update = (
                    f"{segments[i]}: {mask_colors[i]}"
                    if segments
                    else f"Mask {i}: {mask_colors[i]}"
                )
                xbar_title = xbar_title + xbar_title_update

            plt.contour(
                masks_data[i][:, :, current_slice],
                colors=mask_colors[i],
                levels=[0.5],
            )

        plt.title(f"{title} Slice {current_slice} {xbar_title}")
        plt.axis("off")

    ani = animation.FuncAnimation(
        fig, update_slice, frames=slice_range, interval=interval
    )

    if save_path:
        if isinstance(save_path, str):
            save_path = Path(save_path)

        if save_path.suffix != ".png":
            save_path = save_path.with_suffix(".gif")

        if not save_path.parent.exists():
            save_path.parent.mkdir(parents=True)

        ani.save(save_path, writer="pillow")

    plt.show()


def gif_reference(
    image_path,
    mask_reference_path,
    mask_paths,
    segments,
    mask_expert_cmap="jet",
    mask_colors=None,
    interval=100,
    title="Layered Plot",
    save_path="animation.gif",
):
    ct_data = image_path_to_data(image_path)

    mask_reference_data = mask_path_to_data(mask_reference_path)
    slices, _ = viz.significant_slice_idx_data(mask_reference_data)

    masks_data = []
    for mask_path in mask_paths:
        mask_data = mask_path_to_data(mask_path)
        masks_data.append(mask_data)

    num_masks = len(masks_data)

    if segments:
        try:
            assert len(segments) == num_masks
        except AssertionError:
            # raise ValueError("Number of segments must be equal to the number of masks")
            print(
                "Number of segments must be equal to the number of masks, segments will be ignored"
            )
            segments = None

    assert_shape(ct_data + mask_reference_data + masks_data)

    mask_colors = generate_mask_colors(num_masks, mask_colors)

    fig = plt.figure()

    def update_slice(frame):
        global current_slice

        current_slice = frame

        plt.clf()

        plt.imshow(ct_data[:, :, current_slice], cmap="gray")

        plt.imshow(
            mask_reference_data[:, :, current_slice],
            cmap=mask_expert_cmap,
            alpha=0.3,
        )

        xbar_title = ""

        for i in range(len(masks_data)):
            if not np.all(masks_data[i][:, :, current_slice] == False):
                xbar_title_update = (
                    f"{segments[i]}: {mask_colors[i]}"
                    if segments
                    else f"Mask {i}: {mask_colors[i]}"
                )
                xbar_title = xbar_title + xbar_title_update

            plt.contour(
                masks_data[i][:, :, current_slice],
                colors=mask_colors[i],
                levels=[0.5],
            )

        plt.title(f"{title} Slice {current_slice} {xbar_title}")
        plt.axis("off")

    ani = animation.FuncAnimation(fig, update_slice, frames=slices, interval=interval)

    if save_path:
        if isinstance(save_path, str):
            save_path = Path(save_path)

        if save_path.suffix != ".png":
            save_path = save_path.with_suffix(".gif")

        if not save_path.parent.exists():
            save_path.parent.mkdir(parents=True)

        ani.save(save_path, writer="pillow")

    plt.show()



# from .utils.helper_mask import *
# from .plots.helper_plot import *

# # from .preprocess.mask import generate_mask_schema

# # from .plots.layered_plot import *
# # from .plots.layered_plot2D import *
# # from .plots.plot_image import *
# # from .plots.plot3d.plot_images import *
# from .preprocess.mask import generate_mask_schema
# from .preprocess.match_image_mask import match_image_mask

from .utils import (
    image_path_to_data_ax,
    image_path_to_data_sag,
    mask_path_to_data_ax,
    mask_path_to_data_sag,
)

from .utils.array_profile import profile
from .utils.array_threshold import compute_thresholds

from .plots.plot2d.images import (
    images_array as plot2d_images_array,
    images_path as plot2d_images_path,
)

from .plots.plot3d.images import (
    images_array as plot3d_images_array,
    images_path as plot3d_images_path,
)

# from .plots.gif import gif, gif_slices, gif_non, gif_reference



from pathlib import Path

import pandas as pd


def match_image_mask(
    images_path, masks_path, image_func, mask_func, pattern: str, save_path="./"
):
    df_dict = {"ID": [], "Image": [], "Mask": []}

    image_paths = Path(images_path).glob(pattern)
    mask_paths = Path(masks_path).glob(pattern)

    images = {}
    for image_path in image_paths:
        id = int(image_func(image_path.stem))
        image_path = str(image_path)
        if id in images.keys():
            images[id].append(image_path)
        else:
            images[id] = [image_path]

    masks = {}
    for mask_path in mask_paths:
        id = int(mask_func(mask_path.stem))
        mask_path = str(mask_path)
        if id in masks.keys():
            masks[id].append(mask_path)
        else:
            masks[id] = [mask_path]

    # find unique ids
    unique_ids = set(images.keys()).intersection(set(masks.keys()))

    # find all ids
    all_ids = set(images.keys()).union(set(masks.keys()))

    for id in all_ids:
        df_dict["ID"].append(id)

        if id in images.keys():
            df_dict["Image"].append(images[id])
        else:
            df_dict["Image"].append(None)
        if id in masks.keys():
            df_dict["Mask"].append(masks[id])
        else:
            df_dict["Mask"].append(None)

    df = pd.DataFrame(df_dict, columns=df_dict.keys())
    save_path = Path(save_path)
    if not save_path.exists():
        save_path.mkdir(parents=True)

    save_path = save_path / Path("image_mask.csv")
    df.to_csv(save_path, index=False)

    print(f"Number of unique ids: {len(unique_ids)}")
    print(f"Number of all ids: {len(all_ids)}")



import secrets
from pathlib import Path


def assert_shape(arr: list) -> tuple:
    """
    Asserts that all segments have the same shape as the image
    :param arr: list of np arrays
    :return: tuple of the middle slice and the last slice
    usage:
        init_slice, last_slice = assert_shape(ct_data + masks_data)
    """
    # arr[0] is the image
    # arr[1:] are the segments
    # m,n,d are the dimensions of the image
    n, m, d = arr[0].shape
    try:
        for np_array in arr:
            assert np_array.shape == (m, n, d)
    except AssertionError:
        raise ValueError("All segments must have the same shape as the image")

    return d // 2, d - 1


def generate_mask_colors(num_masks, mask_colors=None):
    if mask_colors is None:
        mask_colors = ["#" + secrets.token_hex(3) for _ in range(num_masks)]

        return mask_colors
    else:
        try:
            assert num_masks == len(mask_colors)
            return mask_colors
        except AssertionError:
            raise ValueError(
                f"Number of masks ({num_masks}) does not match number of colors ({len(mask_colors)})"
            )


def plot_image(ax, image_data, cmap="gray"):
    ax.imshow(image_data, cmap=cmap)


def plot_image_imshow(ax, arr, cmap="gray", origin="lower", title="", RGB=False):
    # ax.imshow(arr, cmap=cmap, origin=origin)
    ax.imshow(arr, cmap=cmap)

    ax.set_title(title)


def plot_mask_neighbor(ax, mask_data, cmap="jet", alpha=0.3):
    # ax.imshow(mask_data, cmap=cmap, alpha=alpha, interpolation="bilinear", vmin=0, vmax=1)
    ax.imshow(mask_data, cmap=cmap, alpha=alpha)


# def plot_contour(ax, mask_data, color, line_width=0.5,levels=[0.5]):
#     ax.contour(mask_data, colors=color, linewidths=line_width,levels=levels)
# ax.contour(mask_data, colors=color, linewidths=line_width, levels=[0.5], alpha=0.5)


def plot_contour(ax, mask_data, color, levels=[0.5]):
    ax.contour(mask_data, colors=color, levels=levels)


def save_image(plt, save_path):
    if isinstance(save_path, str):
        save_path = Path(save_path)

    if save_path.suffix != ".png":
        save_path = save_path.with_suffix(".png")

    if not save_path.parent.exists():
        save_path.parent.mkdir(parents=True)

    plt.savefig(save_path)



from pathlib import Path

import matplotlib.pyplot as plt

import medviz as viz

assert_shape = viz.assert_shape
mask_path_to_data = viz.mask_path_to_data_ax
image_path_to_data = viz.image_path_to_data_ax
generate_mask_colors = viz.generate_mask_colors
plot_image = viz.plot_image
plot_contour = viz.plot_contour

significant_slice_idx_data = viz.significant_slice_idx_data


def layered_plot_path2D(
    image_path,
    mask_paths,
    slice,
    mask_colors=None,
    title="Layered Plot",
    save_path=None,
):
    image_data = image_path_to_data(image_path)

    masks_data = []
    for mask_path in mask_paths:
        mask_data = mask_path_to_data(mask_path)
        masks_data.append(mask_data)

    layered_plot_data2D(
        image_data,
        masks_data,
        mask_colors=mask_colors,
        slice=slice,
        title=title,
        save_path=save_path,
    )


def layered_plot_data2D(
    image_data,
    masks_data,
    slice,
    mask_colors=None,
    title="Layered Plot",
    save_path=None,
):
    print("Loading images...")

    num_masks = len(masks_data)

    assert_shape(image_data + masks_data)

    mask_colors = generate_mask_colors(num_masks, mask_colors)

    _, ax = plt.subplots()
    plt.subplots_adjust(bottom=0.25)

    plot_image(ax, image_data[:, :, slice], cmap="gray")

    for i in range(num_masks):
        plot_contour(ax, masks_data[i][:, :, slice], color=mask_colors[i], levels=[0.5])

    ax.set_xlabel(f"Slice Number: {slice}")
    ax.set_title(title)

    plt.show()

    if save_path:
        if isinstance(save_path, str):
            save_path = Path(save_path)

        if save_path.suffix != ".png":
            save_path = save_path.with_suffix(".png")

        if not save_path.parent.exists():
            save_path.parent.mkdir(parents=True)

        plt.savefig(save_path)


import matplotlib.pyplot as plt
import numpy as np


def layered_plot_all2D(
    id, image_data, mask_data, save_path, mask_color="red", title=""
):
    image_data_rot = np.flip(np.rot90(image_data, axes=(1, 0)), axis=1)
    mask_data_rot = np.flip(np.rot90(mask_data, axes=(1, 0)), axis=1)

    most_value_nonzero_slices, num_nonzero_slices = significant_slice_idx_data(
        mask_data_rot
    )
    print(most_value_nonzero_slices, num_nonzero_slices)
    for i in range(num_nonzero_slices):
        slice = most_value_nonzero_slices[i]
        _, ax = plt.subplots()
        plt.subplots_adjust(bottom=0.25)
        plot_image(ax, image_data_rot[:, :, slice], cmap="gray")
        plot_contour(ax, mask_data_rot[:, :, slice], color=mask_color, levels=[0.5])
        ax.set_xlabel(f"ID:{id} Slice Number: {slice}")
        ax.set_title(title)
        plt.tight_layout()

        plt.savefig(f"./crohns_plots/{id}_Slice_{slice}.png")
        plt.close()
    pass


import pandas as pd

if __name__ == "__main__":
    df = pd.read_csv("./crohn/image_mask.csv")
    for index, row in df.iterrows():
        id = row["ID"]
        print(id)
        print(type(id))

        if id != 51:
            image_path = row["Image"].replace("['", "").replace("']", "")
            mask_path = row["Mask"].replace("['", "").replace("']", "")
            print(image_path)
            print(mask_path)

            image_data = np.load(image_path)
            mask_data = np.load(mask_path)
            layered_plot_all2D(id, image_data, mask_data, save_path="layered_plot.png")

            pass
    # image_path = Path("data/image.nii.gz")
    # mask_paths = [Path("data/mask1.nii.gz"), Path("data/mask2.nii.gz")]

    # layered_plot_all2D(image_path, mask_paths,  save_path="layered_plot.png")
    # layered_plot_all2D(image_path, mask_paths,  save_path="crohns_plots")



from pathlib import Path

import matplotlib.pyplot as plt
from matplotlib.widgets import Slider

import medviz as viz

image_path_to_data = viz.image_path_to_data
plot_image = viz.plot_image


def plot_image_path(image_path, title="Image", save_path=None):
    image_data = image_path_to_data(image_path)

    plot_image_data(
        image_data,
        title=title,
        save_path=save_path,
    )


def plot_image_data(image_data, title="Image", save_path=None):
    print("Loading images...")

    d = image_data.shape[2]
    init_slice, last_slice = d // 2, d - 1

    _, ax = plt.subplots()
    plt.subplots_adjust(bottom=0.25)

    plot_image(ax, image_data[:, :, init_slice], cmap="gray")

    ax.set_xlabel(f"Slice Number: {init_slice}")
    ax.set_title(title)

    slider_ax = plt.axes([0.2, 0.1, 0.6, 0.03])

    slider = Slider(
        slider_ax,
        "Slice",
        0,
        last_slice,
        valinit=init_slice,
        valstep=1,
    )

    def update(val):
        slice_num = int(slider.val)
        ax.clear()

        plot_image(ax, image_data[:, :, slice_num], cmap="gray")

        ax.set_xlabel(f"Slice Number: {slice_num}")
        ax.set_title(title)

    slider.on_changed(update)

    plt.show()

    if save_path:
        if isinstance(save_path, str):
            save_path = Path(save_path)

        if save_path.suffix != ".png":
            save_path = save_path.with_suffix(".png")

        if not save_path.parent.exists():
            save_path.parent.mkdir(parents=True)

        plt.savefig(save_path)
